@@ -51,7 +51,7 @@ def credentials_to_profile(credentials: dict) -> dict:
 
 def validate_profile(config: dict, arguments: argparse.Namespace, profiles: dict, target_profile_name: str) -> bool:
     logger.debug('Validating profile')
-    profile = get_profile(config, arguments, profiles, target_profile_name)
+    profile = profiles.get(target_profile_name)
     if arguments.output_profile and not is_mutable_profile(profiles, arguments.output_profile):
         raise exceptions.ImmutableProfileError(arguments.output_profile, 'not awsume-managed')
     if not profile:
@@ -68,10 +68,15 @@ def validate_profile(config: dict, arguments: argparse.Namespace, profiles: dict
         if profile.get('credential_source') not in VALID_CREDENTIAL_SOURCES:
             raise exceptions.InvalidProfileError(target_profile_name, message='unsupported awsume credential_source profile option: {}'.format(profile.get('credential_source')))
         source_profile_name = profile.get('source_profile')
-        if source_profile_name and not profiles.get(source_profile_name):
+        source_profile = get_source_profile(profiles, target_profile_name)
+        if source_profile_name and not source_profile:
             raise exceptions.ProfileNotFoundError(profile_name=source_profile_name)
-        user_profile = get_source_profile(profiles, target_profile_name)
-        user_profile_name = source_profile_name
+        if source_profile and not source_profile.get('role_arn'):
+            user_profile = source_profile
+            user_profile_name = source_profile_name
+        else:
+            user_profile = None
+            user_profile_name = None
     else:
         user_profile = profile
         user_profile_name = target_profile_name
@@ -105,15 +110,6 @@ def is_mutable_profile(profiles: dict, profile_name: str) -> dict:
     return False
 
 
-# def get_autoawsume_profile_name(config: dict, arguments: argparse.Namespace, profiles: dict, profile_name: str) -> dict:
-#     profile = profiles.get(profile_name)
-#     if not profile:
-#         return True
-#     if profile.get('manager') == 'awsume':
-#         return True
-#     return False
-
-
 def get_source_profile(profiles: dict, target_profile_name: str) -> dict:
     target_profile = profiles.get(target_profile_name)
     if target_profile:
@@ -122,6 +118,23 @@ def get_source_profile(profiles: dict, target_profile_name: str) -> dict:
     return None
 
 
+def get_role_chain(profiles: dict, target_profile_name: str) -> list:
+    target_profile = profiles.get(target_profile_name)
+    if not target_profile or not target_profile.get('role_arn'):
+        return [target_profile_name]
+
+    role_chain = []
+    while target_profile:
+        if target_profile.get('role_arn'):
+            if target_profile_name in role_chain:
+                raise exceptions.InvalidProfileError(','.join(role_chain), 'cannot have circular role-chains')
+            role_chain.append(target_profile_name)
+        target_profile_name = target_profile.get('source_profile')
+        target_profile = profiles.get(target_profile_name)
+    role_chain.reverse()
+    return role_chain
+
+
 def get_region(profiles: dict, arguments: argparse.Namespace, config: dict, ignore_config: bool = False, ignore_default: bool = False) -> str:
     if arguments.region:
         return arguments.region
@@ -244,32 +257,30 @@ def get_account_id(profile: dict, call_aws: bool = False) -> str:
     return 'Unavailable'
 
 
-def get_profile(config: dict, arguments: argparse.Namespace, profiles: dict, profile_name: str) -> dict:
-    if profile_name in profiles or not config.get('fuzzy-match'):
-        return profiles.get(profile_name)
-
+def get_profile_name(config: dict, profiles: dict, target_profile_name: str, log=True) -> str:
+    if target_profile_name in profiles or not config.get('fuzzy-match'):
+        return target_profile_name
     profile_names = list(profiles.keys())
 
-    matched_prefix_profile = match_prefix(profile_names, profile_name)
+    matched_prefix_profile = match_prefix(profile_names, target_profile_name)
     if matched_prefix_profile:
-        safe_print('Using profile ' + matched_prefix_profile, color=colorama.Fore.YELLOW)
-        arguments.target_profile_name = matched_prefix_profile
-        return profiles.get(matched_prefix_profile)
+        if log:
+            safe_print('Using profile ' + matched_prefix_profile, color=colorama.Fore.YELLOW)
+        return matched_prefix_profile
 
-    matched_contains_profile = match_contains(profile_names, profile_name)
+    matched_contains_profile = match_contains(profile_names, target_profile_name)
     if matched_contains_profile:
-        safe_print('Using profile ' + matched_contains_profile, color=colorama.Fore.YELLOW)
-        arguments.target_profile_name = matched_contains_profile
-        return profiles.get(matched_contains_profile)
+        if log:
+            safe_print('Using profile ' + matched_contains_profile, color=colorama.Fore.YELLOW)
+        return matched_contains_profile
 
-    matched_levenshtein_profile = match_levenshtein(profile_names, profile_name)
+    matched_levenshtein_profile = match_levenshtein(profile_names, target_profile_name)
     if matched_levenshtein_profile:
-        safe_print('Using profile ' + matched_levenshtein_profile, color=colorama.Fore.YELLOW)
-        arguments.target_profile_name = matched_levenshtein_profile
-        return profiles.get(matched_levenshtein_profile)
+        if log:
+            safe_print('Using profile ' + matched_levenshtein_profile, color=colorama.Fore.YELLOW)
+        return matched_levenshtein_profile
 
-    profile = profiles.get(profile_name)
-    return profile
+    return target_profile_name
 
 
 def match_prefix(profile_names: list, profile_name: str) -> str:
